# -*- coding: utf-8 -*-
def is_enabled():
    try:
        from App.config import getConfiguration
        debug_mode = getConfiguration().debug_mode
    except Exception:
        debug_mode = False
    return debug_mode



# -*- coding: utf-8 -*-
try:
    import ipdb as pdb
except ImportError:
    import pdb

from ZServer.ZPublisher import Publish

real_publish = Publish.publish


def resolveDottedName(dotted_name):
    """Resolve the dotted name importing as necessary then using
    getattr."""
    idx = dotted_name.find('.')
    if idx == -1:
        base_name = dotted_name
        rest_path = []
    else:
        base_name = dotted_name[:idx]
        rest_path = dotted_name[idx + 1:].split('.')

    obj = __import__(base_name)

    for name in rest_path:
        obj = getattr(obj, name)

    return obj


def pdb_runcall(object, args, request):
    """If the request has the pdb_runcall key then we run the result
    of request traversal in the debugger.  Othwise, do it normally.

    A cookie for pdb_runcall may also be set or removed if the request
    has the toggle_runcall key."""
    response = request.response

    if 'toggle_runcall' in request and request.toggle_runcall:
        runcall_cookie = request.cookies.get('pdb_runcall', False)
        if runcall_cookie:
            response.expireCookie('pdb_runcall')
            return Publish.call_object(object, args, request)
        else:
            response.setCookie('pdb_runcall', 1)

    if 'set_runcall_ignore' in request:
        if request.set_runcall_ignore:
            for ignore in request.set_runcall_ignore:
                response.appendCookie('runcall_ignore', ignore)
        else:
            response.expireCookie('runcall_ignore')

    if 'pdb_runcall' in request:
        if request.pdb_runcall:
            ignores = request.get('runcall_ignore', [])
            if ignores:
                ignores = ignores.split(':')
            for ignore in ignores:
                obj = resolveDottedName(ignore)
                if obj.__func__ is getattr(object, 'im_func', None):
                    break
            else:
                return pdb.runcall(object, *args)
    return Publish.call_object(object, args, request)


def pdb_publish(request, module_name, after_list, debug=0,
                call_object=pdb_runcall,
                missing_name=Publish.missing_name,
                dont_publish_class=Publish.dont_publish_class,
                mapply=Publish.mapply, ):
    """Hook the publish function to override the function used to call
    the result of the request traversal."""
    return real_publish(
        request, module_name, after_list, debug=0,
        call_object=call_object,
        missing_name=missing_name,
        dont_publish_class=dont_publish_class,
        mapply=mapply, )



try:
    import ipdb as pdb
except ImportError:
    import pdb


def resolveDottedName(dotted_name):
    """Resolve the dotted name importing as necessary then using
    getattr."""
    idx = dotted_name.find('.')
    if idx == -1:
        base_name = dotted_name
        rest_path = []
    else:
        base_name = dotted_name[:idx]
        rest_path = dotted_name[idx + 1:].split('.')

    obj = __import__(base_name)

    for name in rest_path:
        obj = getattr(obj, name)

    return obj


def pdb_runcall(obj, args, request):
    """If the request has the pdb_runcall key then we run the result
    of request traversal in the debugger.  Othwise, do it normally.

    A cookie for pdb_runcall may also be set or removed if the request
    has the toggle_runcall key."""
    response = request.response

    if 'toggle_runcall' in request and request.toggle_runcall:
        runcall_cookie = request.cookies.get('pdb_runcall', False)
        if runcall_cookie:
            response.expireCookie('pdb_runcall')
            return obj(*args)
        else:
            response.setCookie('pdb_runcall', 1)

    if 'set_runcall_ignore' in request:
        if request.set_runcall_ignore:
            for ignore in request.set_runcall_ignore:
                response.appendCookie('runcall_ignore', ignore)
        else:
            response.expireCookie('runcall_ignore')

    if 'pdb_runcall' in request:
        if request.pdb_runcall:
            ignores = request.get('runcall_ignore', [])
            if ignores:
                ignores = ignores.split(':')
            for ignore in ignores:
                _obj = resolveDottedName(ignore)
                if _obj.__func__ is getattr(obj, 'im_func', None):
                    break
            else:
                return pdb.runcall(obj, *args)
    return obj(*args)



from OFS import Uninstalled

broken = {}
for path, obj in app.ZopeFind(app, search_sub=True):
    if isinstance(obj, Uninstalled.BrokenClass):
        broken.setdefault(obj.__class__, []).append(path)

import pprint
pprint.pprint(broken)



# -*- coding: utf-8 -*-
try:
    import ipdb as pdb
except ImportError:
    import pdb

from Products.SiteErrorLog import SiteErrorLog

from Products.PDBDebugMode import pdblogging

orig_raising = SiteErrorLog.SiteErrorLog.raising


def raising(self, info):
    """Catch the traceback and bypass pdblogging"""
    def error(msg, *args, **kw):
        return pdblogging.orig_error(
            SiteErrorLog.LOG, msg, *args, **kw)
    SiteErrorLog.LOG.error = error
    result = orig_raising(self, info)
    if result:
        pdb.post_mortem(info[2])
    del SiteErrorLog.LOG.error
    return result



from ZODB import Connection
from ZODB.serialize import ObjectWriter

orig_register = Connection.Connection.register


def register(self, obj):
    """
    Serialize early to inspect PicklingErrors

    Raise any PicklingErrors when the object is added to the
    transaction as opposed to when the transaction is committed.
    Under pdb, for example, this allows inspecting the code that made
    the change resulting in the PicklingError.

    Requires either zope.testrunner or zope.testing which can be included using
    the 'zodb' or 'zodb-testing' extras respectively.
    """
    orig_register(self, obj)

    writer = ObjectWriter(obj)

    # Replace the pickler so that it doesn't set oids
    import six.moves.cPickle as pickle
    writer._p = pickle.Pickler(writer._file, 1)

    # Try to serialize to raise piclkling errors early
    writer.serialize(obj)



"""If debug-mode=on enable import of pdb (and ipdb) in
unprotected code.
"""
from AccessControl import allow_module
from Products.PDBDebugMode import debugger
from types import ModuleType

import sys
import logging

log = logging.getLogger(__name__)


if debugger.is_enabled():
    enabled = ModuleType('enabled')
    sys.modules['Products.PDBDebugMode.enabled'] = enabled
    warning = ("""

******************************************************************************

Debug-Mode enabled!

This will result in a pdb when a exception happens.
Turn off debug mode or remove Products.PDBDebugMode to disable.

See https://pypi.python.org/pypi/Products.PDBDebugMode

******************************************************************************
""")

    log.warning(warning)
    # Allow import of pdb in unprotected code
    allow_module('pdb')
    allow_module('ipdb')
    allow_module('Products.PDBDebugMode.debugger')
